@@ -8,6 +8,7 @@
 
 use criterion::{criterion_group, criterion_main, Criterion};
 use eyre::Result;
+use safe_network::client::client_api::Blob;
 use safe_network::url::Scope;
 use safe_network::{
     client::{
@@ -21,10 +22,12 @@ use tokio::runtime::Runtime;
 /// This bench requires a network already set up
 async fn upload_bytes(size: usize) -> Result<(), Error> {
     let (genesis_key, bootstrap_nodes) = read_network_conn_info().unwrap();
-    let data = random_bytes(size);
+    let blob = Blob::new(random_bytes(size))?;
     let config = Config::new(None, None, genesis_key, None, None).await;
     let client = Client::new(config, bootstrap_nodes, None).await?;
-    let address = client.write_to_network(data.clone(), Scope::Public).await?;
+    let address = client
+        .write_blob_to_network(blob.clone(), Scope::Public)
+        .await?;
 
     // the larger the file, the longer we have to wait before we start querying
     let delay = usize::max(1, size / 2_000_000);
@@ -32,7 +35,7 @@ async fn upload_bytes(size: usize) -> Result<(), Error> {
     // let's make sure the public chunk is stored
     let received_data = run_w_backoff_delayed(|| client.read_blob(address), 10, delay).await?;
 
-    assert_eq!(received_data, data);
+    assert_eq!(received_data, blob.bytes());
 
     Ok(())
 }
