@@ -15,14 +15,12 @@ use crate::messaging::{
     system::{KeyedSig, SectionAuth, SystemMsg},
     DstLocation, MessageId, MessageType, MsgKind, SectionAuthorityProvider, WireMsg,
 };
-use crate::routing::ELDER_SIZE;
-use crate::types::PublicKey;
+use crate::messaging::{AuthorityProof, ServiceAuth};
 use bytes::Bytes;
 use itertools::Itertools;
 use qp2p::IncomingMessages;
 use secured_linked_list::SecuredLinkedList;
 use std::net::SocketAddr;
-use xor_name::XorName;
 
 impl Session {
     // Listen for incoming messages on a connection
@@ -33,7 +31,7 @@ impl Session {
         debug!("Listening for incoming messages");
         let _ = tokio::spawn(async move {
             loop {
-                session = match Self::get_incoming_message(&mut incoming_messages).await {
+                session = match Self::listen_for_incoming_message(&mut incoming_messages).await {
                     Ok((src, msg)) => match Self::handle_msg(msg, src, session.clone()).await {
                         Ok(session) => session,
                         Err(err) => {
@@ -55,12 +53,14 @@ impl Session {
         });
     }
 
-    pub(crate) async fn get_incoming_message(
+    pub(crate) async fn listen_for_incoming_message(
         incoming_messages: &mut IncomingMessages,
     ) -> Result<(SocketAddr, MessageType), Error> {
         if let Some((src, message)) = incoming_messages.next().await {
-            let msg_type = WireMsg::deserialize(message)?;
             trace!("Incoming message from {:?}", &src);
+            let msg_type = WireMsg::deserialize(message)?;
+            trace!("Incoming message deserialized fine");
+
             Ok((src, msg_type))
         } else {
             Err(Error::Generic("Nothing..".to_string())) // TODO: FIX error type
@@ -72,6 +72,7 @@ impl Session {
         src: SocketAddr,
         session: Session,
     ) -> Result<Session, Error> {
+        trace!("client handle msg afoot");
         match msg {
             MessageType::Service { msg_id, msg, .. } => {
                 Self::handle_client_msg(session, msg_id, msg, src).await
@@ -195,78 +196,70 @@ impl Session {
     // Handle Anti-Entropy Redirect messages
     async fn handle_ae_redirect_msg(
         session: Session,
-        section_auth: SectionAuthorityProvider,
+        target_section_auth: SectionAuthorityProvider,
         section_signed: KeyedSig,
         bounced_msg: Bytes,
         sender: SocketAddr,
     ) -> Result<Session, Error> {
         // Check if SAP signature is valid
-        if !bincode::serialize(&section_auth)
+        if !bincode::serialize(&target_section_auth)
             .map(|bytes| section_signed.verify(&bytes))
             .unwrap_or(false)
         {
             warn!(
                 "Signature returned with SAP in AE-Redirect response is invalid: {:?}",
-                section_auth
+                target_section_auth
             );
             return Ok(session);
         }
 
-        let mut num_of_elders_for_query = ELDER_SIZE;
+        debug!(
+            "Received AE-Redirect for from {}, with SAP: {:?}",
+            sender, target_section_auth
+        );
 
-        let (msg_id, service_msg, auth) = match WireMsg::deserialize(bounced_msg)? {
-            MessageType::Service {
-                msg_id, msg, auth, ..
-            } => {
-                if let ServiceMsg::Query(_) = msg {
-                    num_of_elders_for_query = NUM_OF_ELDERS_SUBSET_FOR_QUERIES;
-                }
-                (msg_id, msg, auth)
-            }
-            other => {
-                warn!(
-                    "Unexpected non-serviceMsg returned in AE-Redirect response: {:?}",
-                    other
-                );
+        if let Some((session, elders, service_msg, mut dst_location, auth)) =
+            Self::new_elder_targets_if_any(
+                session.clone(),
+                bounced_msg.clone(),
+                &target_section_auth,
+            )
+            .await?
+        {
+            if elders.is_empty() {
+                debug!("We have already resent this message on an AE-Redirect response. Dropping this instance");
                 return Ok(session);
             }
-        };
-        debug!(
-            "Received AE-Redirect for {:?}, from {}, with SAP: {:?}",
-            msg_id, sender, section_auth
-        );
 
-        debug!(
-            "Bounced message ({:?}) received in AE-Redirect response: {:?}",
-            msg_id, service_msg
-        );
+            let message = WireMsg::serialize_msg_payload(&service_msg)?;
 
-        let message = WireMsg::serialize_msg_payload(&service_msg)?;
+            // We cannot trust these Elders belong to the network we are intended to connect to (based on the genesis key we know).
+            // Here we purposefully try and use the genesis key trigger an AE-Retry and expand our network knowledge.
 
-        // TODO: we cannot trust these Elders belong to the network we are intended
-        // to connect to (based on the genesis key we know). We could send the genesis key
-        // as the destination section key and that should cause an AE-Retry response,
-        // which we could use to verify the SAP we receive an trust.
-        let elders = section_auth
-            .elders
-            .values()
-            .cloned()
-            .take(num_of_elders_for_query)
-            .collect::<Vec<SocketAddr>>();
-        let section_pk = section_auth.public_key_set.public_key();
+            // TODO: can we attempt to verify the key against out knowledge? (was it signed by a key we know?)
+            let section_pk = session.genesis_key;
+            debug!("Genesis PK we'll be sending is: {:?}", section_pk);
 
-        // Let's rebuild the message with the updated destination details
-        let wire_msg = WireMsg::new_msg(
-            msg_id,
-            message,
-            MsgKind::ServiceMsg(auth.into_inner()),
-            DstLocation::Section {
-                name: XorName::from(PublicKey::Bls(section_pk)),
+            dst_location.set_section_pk(section_pk);
+
+            let msg_id = MessageId::new();
+            // Let's rebuild the message with the updated destination details
+            let wire_msg = WireMsg::new_msg(
+                msg_id,
+                message,
+                MsgKind::ServiceMsg(auth.into_inner()),
+                dst_location,
+            )?;
+
+            debug!("Resending original message on AE-Redirect with updated details. Expecting an AE-Retry next");
+            send_message(elders.clone(), wire_msg, session.endpoint.clone(), msg_id).await?;
+
+            let _old_entry = session.ae_cache.write().await.insert((
+                dst_location.name(),
                 section_pk,
-            },
-        )?;
-
-        send_message(elders, wire_msg, session.endpoint.clone(), msg_id).await?;
+                bounced_msg,
+            ));
+        }
 
         Ok(session)
     }
@@ -279,13 +272,94 @@ impl Session {
         bounced_msg: Bytes,
         proof_chain: SecuredLinkedList,
     ) -> Result<Session, Error> {
-        // Remove expired items from ae_cache before checking.
-        // It might be late to not retry now.
-        session.ae_cache.remove_expired().await;
+        // Extract necessary information for resending
+        if let Some((session, elders, service_msg, mut dst_location, auth)) =
+            Self::new_elder_targets_if_any(session.clone(), bounced_msg.clone(), &section_auth)
+                .await?
+        {
+            debug!("Received AE-Retry with new SAP: {:?}", section_auth);
 
-        // Deserialize the bounced message for resending
-        let (msg_id, service_msg, mut dst_location, auth): (_, ServiceMsg, _, _) =
-            match WireMsg::deserialize(bounced_msg)? {
+            if elders.is_empty() {
+                debug!("We have already responded to this message on an AE-Retry response. Dropping this instance");
+                return Ok(session);
+            }
+            // Update our network knowledge making sure proof chain
+            // validates the new SAP based on currently known remote section SAP.
+            match session.network.update(
+                SectionAuth {
+                    value: section_auth.clone(),
+                    sig: section_signed,
+                },
+                &proof_chain,
+            ) {
+                Ok(updated) => {
+                    if updated {
+                        debug!(
+                            "Anti-Entropy: updated remote section SAP updated for {:?}",
+                            section_auth.prefix
+                        );
+                    } else {
+                        debug!(
+                                "Anti-Entropy: discarded SAP for {:?} since it's the same as the one in our records: {:?}",
+                                section_auth.prefix, section_auth
+                            );
+                    }
+                }
+                Err(err) => {
+                    warn!(
+                            "Anti-Entropy: failed to update remote section SAP, bounced msg dropped: {:?}, {:?}",
+                            err, service_msg
+                        );
+                    return Ok(session);
+                }
+            }
+
+            let payload = WireMsg::serialize_msg_payload(&service_msg)?;
+
+            let target_section_pk = section_auth.public_key_set.public_key();
+            // Let's rebuild the message with the updated destination details
+            dst_location.set_section_pk(target_section_pk);
+
+            let msg_id = MessageId::new();
+
+            let wire_msg = WireMsg::new_msg(
+                msg_id,
+                payload,
+                MsgKind::ServiceMsg(auth.into_inner()),
+                dst_location,
+            )?;
+
+            debug!("Resending original message via AE-Retry");
+
+            send_message(elders.clone(), wire_msg, session.endpoint.clone(), msg_id).await?;
+
+            let _old_entry = session.ae_cache.write().await.insert((
+                dst_location.name(),
+                target_section_pk,
+                bounced_msg,
+            ));
+        }
+
+        Ok(session)
+    }
+
+    /// Checks AE cache to see if we should be forwarding this message (and to whom) or if it has already been dealt with
+    async fn new_elder_targets_if_any(
+        session: Session,
+        bounced_msg: Bytes,
+        received_auth: &SectionAuthorityProvider,
+    ) -> Result<
+        Option<(
+            Session,
+            Vec<SocketAddr>,
+            ServiceMsg,
+            DstLocation,
+            AuthorityProof<ServiceAuth>,
+        )>,
+        Error,
+    > {
+        let (msg_id, service_msg, dst_location, auth) =
+            match WireMsg::deserialize(bounced_msg.clone())? {
                 MessageType::Service {
                     msg_id,
                     msg,
@@ -294,14 +368,19 @@ impl Session {
                 } => (msg_id, msg, dst_location, auth),
                 other => {
                     warn!(
-                        "Unexpected non-serviceMsg returned in AE response: {:?}",
+                        "Unexpected non-serviceMsg returned in AE-Redirect response: {:?}",
                         other
                     );
-                    return Ok(session);
+                    return Ok(None);
                 }
             };
 
-        let (targets, dst_address_of_bounced_msg) = match &service_msg {
+        debug!(
+            "Bounced message ({:?}) received in an AE response: {:?}",
+            msg_id, service_msg
+        );
+
+        let (target_count, dst_address_of_bounced_msg) = match service_msg.clone() {
             ServiceMsg::Cmd(cmd) => {
                 match &cmd {
                     DataCmd::StoreChunk(_) => (3, cmd.dst_name()), // stored at Adults, so only 1 correctly functioning Elder need to relay
@@ -311,95 +390,55 @@ impl Session {
             ServiceMsg::Query(query) => (NUM_OF_ELDERS_SUBSET_FOR_QUERIES, query.dst_name()),
             _ => {
                 warn!(
-                    "Bounced message ({:?}) received in AE response: {:?} is of invalid type",
-                    msg_id, service_msg
+                    "Invalid bounced message received in AE response: {:?}. Message is of invalid type",
+                    service_msg
                 );
-                return Ok(session);
+                // Early return with random name as we will discard the message at the caller func
+                return Ok(None);
             }
         };
 
-        if let Some(old_elders) = session.ae_cache.get(&dst_address_of_bounced_msg).await {
-            let received_elders = section_auth
-                .elders
-                .values()
-                .cloned()
-                .collect::<Vec<SocketAddr>>();
-            if old_elders == received_elders {
-                debug!("We have already resent this message on a AE-Retry. Dropping this instance");
-                return Ok(session);
-            }
-        }
-
-        debug!(
-            "Received AE-Retry for msg_id: {:?} with new SAP: {:?}",
-            msg_id, section_auth
-        );
-        // Update our network knowledge making sure proof chain
-        // validates the new SAP based on currently known remote section SAP.
-        match session.network.update(
-            SectionAuth {
-                value: section_auth.clone(),
-                sig: section_signed,
-            },
-            &proof_chain,
-        ) {
-            Ok(updated) => {
-                if updated {
-                    debug!(
-                        "Anti-Entropy: updated remote section SAP updated for {:?}",
-                        section_auth.prefix
-                    );
-                } else {
-                    debug!(
-                        "Anti-Entropy: discarded SAP for {:?} since it's the same as the one in our records: {:?}",
-                        section_auth.prefix, section_auth
-                    );
-                }
-            }
-            Err(err) => {
-                warn!(
-                    "Anti-Entropy: failed to update remote section SAP, bounced msg dropped: {:?}",
-                    err
-                );
-                return Ok(session);
-            }
-        }
-
-        debug!(
-            "Bounced message ({:?}) received in AE response: {:?}",
-            msg_id, service_msg
-        );
-        let payload = WireMsg::serialize_msg_payload(&service_msg)?;
-
-        // Let's rebuild the message with the updated destination details
-        let elders = section_auth
+        let mut target_elders = received_auth
             .elders
-            .into_iter()
+            .iter()
             .sorted_by(|(lhs_name, _), (rhs_name, _)| {
                 dst_address_of_bounced_msg.cmp_distance(lhs_name, rhs_name)
             })
             .map(|(_, addr)| addr)
-            .take(targets)
+            .take(target_count)
+            .cloned()
             .collect::<Vec<SocketAddr>>();
 
-        dst_location.set_section_pk(section_auth.public_key_set.public_key());
+        let mut the_cache_guard = session.ae_cache.write().await;
 
-        let wire_msg = WireMsg::new_msg(
-            msg_id,
-            payload,
-            MsgKind::ServiceMsg(auth.into_inner()),
-            dst_location,
-        )?;
+        let cache_entry = the_cache_guard.find(|x| {
+            x == &(
+                dst_address_of_bounced_msg,
+                received_auth.public_key_set.public_key(),
+                bounced_msg.clone(),
+            )
+        });
 
-        send_message(elders.clone(), wire_msg, session.endpoint.clone(), msg_id).await?;
-        if let Some(old_elders) = session
-            .ae_cache
-            .set(dst_address_of_bounced_msg, elders.clone(), None)
-            .await
-        {
-            warn!("We have already sent this message to Elders {:?} Updating cache with latest elders {:?}", old_elders, &elders);
+        if cache_entry.is_some() {
+            // an elder group corresponds to a PK, so as we've sent to this PK, we've sent to these elders...
+            debug!("Cache hit! We have sent this message before, removing target elders");
+
+            target_elders = vec![];
         }
 
-        Ok(session)
+        debug!(
+            "Final target elders for resending {:?} message are {:?}",
+            service_msg, target_elders
+        );
+
+        drop(the_cache_guard);
+
+        Ok(Some((
+            session,
+            target_elders,
+            service_msg,
+            dst_location,
+            auth,
+        )))
     }
 }
